@@ -17,6 +17,7 @@
 
 #include <arrow/io/file.h>
 #include <arrow/util/logging.h>
+#include <arrow/util/secure_string.h>
 #include <dirent.h>
 #include <parquet/api/reader.h>
 #include <parquet/api/writer.h>
@@ -92,9 +93,9 @@
 
 constexpr int NUM_ROWS_PER_ROW_GROUP = 500;
 
-const char* kFooterEncryptionKey = "0123456789012345";  // 128bit/16
-const char* kColumnEncryptionKey1 = "1234567890123450";
-const char* kColumnEncryptionKey2 = "1234567890123451";
+const arrow::util::SecureString kFooterEncryptionKey("0123456789012345");
+const arrow::util::SecureString kColumnEncryptionKey1("1234567890123450");
+const arrow::util::SecureString kColumnEncryptionKey2("1234567890123451");
 const char* fileName = "tester";
 
 using FileClass = ::arrow::io::FileOutputStream;
@@ -419,7 +420,7 @@ void InteropTestReadEncryptedParquetFiles(std::string root_path) {
   for (unsigned example_id = 0; example_id < vector_of_decryption_configurations.size();
        ++example_id) {
     PrintDecryptionConfiguration(example_id + 1);
-    for (auto const& file : files_in_directory) {
+    for (const auto& file : files_in_directory) {
       std::string exception_msg = "";
       if (!FileNameEndsWith(file, "parquet.encrypted"))  // Skip non encrypted files
         continue;
