@@ -17,6 +17,7 @@
 
 #include "arrow/csv/converter.h"
 
+#include <array>
 #include <cstring>
 #include <limits>
 #include <sstream>
@@ -35,8 +36,8 @@
 #include "arrow/type_traits.h"
 #include "arrow/util/checked_cast.h"
 #include "arrow/util/decimal.h"
-#include "arrow/util/trie.h"
-#include "arrow/util/utf8.h"
+#include "arrow/util/trie_internal.h"
+#include "arrow/util/utf8_internal.h"
 #include "arrow/util/value_parsing.h"  // IWYU pragma: keep
 
 namespace arrow {
@@ -95,16 +96,13 @@ Status InitializeTrie(const std::vector<std::string>& inputs, Trie* trie) {
 
 // Presize a builder based on parser contents
 template <typename BuilderType>
-enable_if_t<!is_base_binary_type<typename BuilderType::TypeClass>::value, Status>
-PresizeBuilder(const BlockParser& parser, BuilderType* builder) {
-  return builder->Resize(parser.num_rows());
-}
-
-// Same, for variable-sized binary builders
-template <typename T>
-Status PresizeBuilder(const BlockParser& parser, BaseBinaryBuilder<T>* builder) {
+Status PresizeBuilder(const BlockParser& parser, BuilderType* builder) {
   RETURN_NOT_OK(builder->Resize(parser.num_rows()));
-  return builder->ReserveData(parser.num_bytes());
+  if constexpr (is_base_binary_type<typename BuilderType::TypeClass>::value) {
+    return builder->ReserveData(parser.num_bytes());
+  } else {
+    return Status::OK();
+  }
 }
 
 /////////////////////////////////////////////////////////////////////////
@@ -121,16 +119,16 @@ struct ValueDecoder {
   }
 
   bool IsNull(const uint8_t* data, uint32_t size, bool quoted) {
-    if (quoted) {
+    if (quoted && !options_.quoted_strings_can_be_null) {
       return false;
     }
-    return null_trie_.Find(
-               util::string_view(reinterpret_cast<const char*>(data), size)) >= 0;
+    return null_trie_.Find(std::string_view(reinterpret_cast<const char*>(data), size)) >=
+           0;
   }
 
  protected:
   Trie null_trie_;
-  std::shared_ptr<DataType> type_;
+  const std::shared_ptr<DataType> type_;
   const ConvertOptions& options_;
 };
 
@@ -165,7 +163,7 @@ struct FixedSizeBinaryValueDecoder : public ValueDecoder {
 
 template <bool CheckUTF8>
 struct BinaryValueDecoder : public ValueDecoder {
-  using value_type = util::string_view;
+  using value_type = std::string_view;
 
   using ValueDecoder::ValueDecoder;
 
@@ -175,7 +173,7 @@ struct BinaryValueDecoder : public ValueDecoder {
   }
 
   Status Decode(const uint8_t* data, uint32_t size, bool quoted, value_type* out) {
-    if (CheckUTF8 && ARROW_PREDICT_FALSE(!util::ValidateUTF8(data, size))) {
+    if (CheckUTF8 && ARROW_PREDICT_FALSE(!util::ValidateUTF8Inline(data, size))) {
       return Status::Invalid("CSV conversion error to ", type_->ToString(),
                              ": invalid UTF8 data");
     }
@@ -191,24 +189,42 @@ struct BinaryValueDecoder : public ValueDecoder {
 };
 
 //
-// Value decoder for integers and floats
+// Value decoder for integers, floats and temporals
 //
 
+template <typename T>
+static arrow::internal::StringConverter<T> MakeStringConverter(
+    const ConvertOptions& options) {
+  if constexpr (is_floating_type<T>::value) {
+    return arrow::internal::StringConverter<T>{options.decimal_point};
+  } else {
+    return arrow::internal::StringConverter<T>{};
+  }
+}
+
 template <typename T>
 struct NumericValueDecoder : public ValueDecoder {
   using value_type = typename T::c_type;
 
-  using ValueDecoder::ValueDecoder;
+  NumericValueDecoder(const std::shared_ptr<DataType>& type,
+                      const ConvertOptions& options)
+      : ValueDecoder(type, options),
+        concrete_type_(checked_cast<const T&>(*type)),
+        string_converter_(MakeStringConverter<T>(options)) {}
 
   Status Decode(const uint8_t* data, uint32_t size, bool quoted, value_type* out) {
     // XXX should quoted values be allowed at all?
     TrimWhiteSpace(&data, &size);
-    if (ARROW_PREDICT_FALSE(
-            !internal::ParseValue<T>(reinterpret_cast<const char*>(data), size, out))) {
+    if (ARROW_PREDICT_FALSE(!string_converter_.Convert(
+            concrete_type_, reinterpret_cast<const char*>(data), size, out))) {
       return GenericConversionError(type_, data, size);
     }
     return Status::OK();
   }
+
+ protected:
+  const T& concrete_type_;
+  arrow::internal::StringConverter<T> string_converter_;
 };
 
 //
@@ -229,12 +245,12 @@ struct BooleanValueDecoder : public ValueDecoder {
 
   Status Decode(const uint8_t* data, uint32_t size, bool quoted, value_type* out) {
     // XXX should quoted values be allowed at all?
-    if (false_trie_.Find(util::string_view(reinterpret_cast<const char*>(data), size)) >=
+    if (false_trie_.Find(std::string_view(reinterpret_cast<const char*>(data), size)) >=
         0) {
       *out = false;
       return Status::OK();
     }
-    if (ARROW_PREDICT_TRUE(true_trie_.Find(util::string_view(
+    if (ARROW_PREDICT_TRUE(true_trie_.Find(std::string_view(
                                reinterpret_cast<const char*>(data), size)) >= 0)) {
       *out = true;
       return Status::OK();
@@ -265,7 +281,7 @@ struct DecimalValueDecoder : public ValueDecoder {
     TrimWhiteSpace(&data, &size);
     Decimal128 decimal;
     int32_t precision, scale;
-    util::string_view view(reinterpret_cast<const char*>(data), size);
+    std::string_view view(reinterpret_cast<const char*>(data), size);
     RETURN_NOT_OK(Decimal128::FromString(view, &decimal, &precision, &scale));
     if (precision > type_precision_) {
       return Status::Invalid("Error converting '", view, "' to ", type_->ToString(),
@@ -285,6 +301,54 @@ struct DecimalValueDecoder : public ValueDecoder {
   const int32_t type_scale_;
 };
 
+//
+// Value decoder wrapper for decimals with a non-default decimal point
+//
+
+template <typename WrappedDecoder>
+struct CustomDecimalPointValueDecoder : public ValueDecoder {
+  using value_type = typename WrappedDecoder::value_type;
+
+  explicit CustomDecimalPointValueDecoder(const std::shared_ptr<DataType>& type,
+                                          const ConvertOptions& options)
+      : ValueDecoder(type, options), wrapped_decoder_(type, options) {}
+
+  Status Initialize() {
+    RETURN_NOT_OK(wrapped_decoder_.Initialize());
+    for (int i = 0; i < 256; ++i) {
+      mapping_[i] = i;
+    }
+    mapping_[options_.decimal_point] = '.';
+    mapping_['.'] = options_.decimal_point;  // error out on standard decimal point
+    temp_.resize(30);
+    return Status::OK();
+  }
+
+  Status Decode(const uint8_t* data, uint32_t size, bool quoted, value_type* out) {
+    if (ARROW_PREDICT_FALSE(size > temp_.size())) {
+      temp_.resize(size);
+    }
+    uint8_t* temp_data = temp_.data();
+    for (uint32_t i = 0; i < size; ++i) {
+      temp_data[i] = mapping_[data[i]];
+    }
+    if (ARROW_PREDICT_FALSE(
+            !wrapped_decoder_.Decode(temp_data, size, quoted, out).ok())) {
+      return GenericConversionError(type_, data, size);
+    }
+    return Status::OK();
+  }
+
+  bool IsNull(const uint8_t* data, uint32_t size, bool quoted) {
+    return wrapped_decoder_.IsNull(data, size, quoted);
+  }
+
+ protected:
+  WrappedDecoder wrapped_decoder_;
+  std::array<uint8_t, 256> mapping_;
+  std::vector<uint8_t> temp_;
+};
+
 //
 // Value decoders for timestamps
 //
@@ -295,18 +359,37 @@ struct InlineISO8601ValueDecoder : public ValueDecoder {
   explicit InlineISO8601ValueDecoder(const std::shared_ptr<DataType>& type,
                                      const ConvertOptions& options)
       : ValueDecoder(type, options),
-        unit_(checked_cast<const TimestampType&>(*type_).unit()) {}
+        unit_(checked_cast<const TimestampType&>(*type_).unit()),
+        expect_timezone_(!checked_cast<const TimestampType&>(*type_).timezone().empty()) {
+  }
 
   Status Decode(const uint8_t* data, uint32_t size, bool quoted, value_type* out) {
-    if (ARROW_PREDICT_FALSE(!internal::ParseTimestampISO8601(
-            reinterpret_cast<const char*>(data), size, unit_, out))) {
+    bool zone_offset_present = false;
+    if (ARROW_PREDICT_FALSE(
+            !internal::ParseTimestampISO8601(reinterpret_cast<const char*>(data), size,
+                                             unit_, out, &zone_offset_present))) {
       return GenericConversionError(type_, data, size);
     }
+    if (zone_offset_present != expect_timezone_) {
+      if (expect_timezone_) {
+        return Status::Invalid("CSV conversion error to ", type_->ToString(),
+                               ": expected a zone offset in '",
+                               std::string(reinterpret_cast<const char*>(data), size),
+                               "'. If these timestamps are in local time, parse them as "
+                               "timestamps without timezone, then call assume_timezone.");
+      } else {
+        return Status::Invalid("CSV conversion error to ", type_->ToString(),
+                               ": expected no zone offset in '",
+                               std::string(reinterpret_cast<const char*>(data), size),
+                               "'");
+      }
+    }
     return Status::OK();
   }
 
  protected:
   TimeUnit::type unit_;
+  bool expect_timezone_;
 };
 
 struct SingleParserTimestampValueDecoder : public ValueDecoder {
@@ -316,18 +399,36 @@ struct SingleParserTimestampValueDecoder : public ValueDecoder {
                                              const ConvertOptions& options)
       : ValueDecoder(type, options),
         unit_(checked_cast<const TimestampType&>(*type_).unit()),
+        expect_timezone_(!checked_cast<const TimestampType&>(*type_).timezone().empty()),
         parser_(*options_.timestamp_parsers[0]) {}
 
   Status Decode(const uint8_t* data, uint32_t size, bool quoted, value_type* out) {
-    if (ARROW_PREDICT_FALSE(
-            !parser_(reinterpret_cast<const char*>(data), size, unit_, out))) {
+    bool zone_offset_present = false;
+    if (ARROW_PREDICT_FALSE(!parser_(reinterpret_cast<const char*>(data), size, unit_,
+                                     out, &zone_offset_present))) {
       return GenericConversionError(type_, data, size);
     }
+    if (zone_offset_present != expect_timezone_) {
+      if (expect_timezone_) {
+        return Status::Invalid("CSV conversion error to ", type_->ToString(),
+                               ": expected a zone offset in '",
+                               std::string(reinterpret_cast<const char*>(data), size),
+                               "'. If these timestamps are in local time, parse them as "
+                               "timestamps without timezone, then call assume_timezone. "
+                               "If using strptime, ensure '%z' is in the format string.");
+      } else {
+        return Status::Invalid("CSV conversion error to ", type_->ToString(),
+                               ": expected no zone offset in '",
+                               std::string(reinterpret_cast<const char*>(data), size),
+                               "'");
+      }
+    }
     return Status::OK();
   }
 
  protected:
   TimeUnit::type unit_;
+  bool expect_timezone_;
   const TimestampParser& parser_;
 };
 
@@ -338,11 +439,15 @@ struct MultipleParsersTimestampValueDecoder : public ValueDecoder {
                                                 const ConvertOptions& options)
       : ValueDecoder(type, options),
         unit_(checked_cast<const TimestampType&>(*type_).unit()),
+        expect_timezone_(!checked_cast<const TimestampType&>(*type_).timezone().empty()),
         parsers_(GetParsers(options_)) {}
 
   Status Decode(const uint8_t* data, uint32_t size, bool quoted, value_type* out) {
+    bool zone_offset_present = false;
     for (const auto& parser : parsers_) {
-      if (parser->operator()(reinterpret_cast<const char*>(data), size, unit_, out)) {
+      if (parser->operator()(reinterpret_cast<const char*>(data), size, unit_, out,
+                             &zone_offset_present) &&
+          zone_offset_present == expect_timezone_) {
         return Status::OK();
       }
     }
@@ -361,9 +466,36 @@ struct MultipleParsersTimestampValueDecoder : public ValueDecoder {
   }
 
   TimeUnit::type unit_;
+  bool expect_timezone_;
   std::vector<const TimestampParser*> parsers_;
 };
 
+//
+// Value decoder for durations
+//
+struct DurationValueDecoder : public ValueDecoder {
+  using value_type = int64_t;
+
+  explicit DurationValueDecoder(const std::shared_ptr<DataType>& type,
+                                const ConvertOptions& options)
+      : ValueDecoder(type, options),
+        concrete_type_(checked_cast<const DurationType&>(*type)),
+        string_converter_() {}
+
+  Status Decode(const uint8_t* data, uint32_t size, bool quoted, value_type* out) {
+    TrimWhiteSpace(&data, &size);
+    if (ARROW_PREDICT_FALSE(!string_converter_.Convert(
+            concrete_type_, reinterpret_cast<const char*>(data), size, out))) {
+      return GenericConversionError(type_, data, size);
+    }
+    return Status::OK();
+  }
+
+ protected:
+  const DurationType& concrete_type_;
+  arrow::internal::StringConverter<DurationType> string_converter_;
+};
+
 /////////////////////////////////////////////////////////////////////////
 // Concrete Converter hierarchy
 
@@ -527,6 +659,24 @@ std::shared_ptr<Converter> MakeTimestampConverter(const std::shared_ptr<DataType
   }
 }
 
+//
+// Concrete Converter factory for reals
+//
+
+template <typename ConverterType, template <typename...> class ConcreteConverterType,
+          typename Type, typename DecoderType>
+std::shared_ptr<ConverterType> MakeRealConverter(const std::shared_ptr<DataType>& type,
+                                                 const ConvertOptions& options,
+                                                 MemoryPool* pool) {
+  if (options.decimal_point == '.') {
+    return std::make_shared<ConcreteConverterType<Type, DecoderType>>(type, options,
+                                                                      pool);
+  }
+  return std::make_shared<
+      ConcreteConverterType<Type, CustomDecimalPointValueDecoder<DecoderType>>>(
+      type, options, pool);
+}
+
 }  // namespace
 
 /////////////////////////////////////////////////////////////////////////
@@ -556,6 +706,12 @@ Result<std::shared_ptr<Converter>> Converter::Make(const std::shared_ptr<DataTyp
   CONVERTER_CASE(TYPE_ID,                           \
                  (PrimitiveConverter<TYPE_CLASS, NumericValueDecoder<TYPE_CLASS>>))
 
+#define REAL_CONVERTER_CASE(TYPE_ID, TYPE_CLASS, DECODER)                        \
+  case TYPE_ID:                                                                  \
+    ptr = MakeRealConverter<Converter, PrimitiveConverter, TYPE_CLASS, DECODER>( \
+        type, options, pool);                                                    \
+    break;
+
     CONVERTER_CASE(Type::NA, NullConverter)
     NUMERIC_CONVERTER_CASE(Type::INT8, Int8Type)
     NUMERIC_CONVERTER_CASE(Type::INT16, Int16Type)
@@ -567,8 +723,12 @@ Result<std::shared_ptr<Converter>> Converter::Make(const std::shared_ptr<DataTyp
     NUMERIC_CONVERTER_CASE(Type::UINT64, UInt64Type)
     NUMERIC_CONVERTER_CASE(Type::FLOAT, FloatType)
     NUMERIC_CONVERTER_CASE(Type::DOUBLE, DoubleType)
+    REAL_CONVERTER_CASE(Type::DECIMAL, Decimal128Type, DecimalValueDecoder)
     NUMERIC_CONVERTER_CASE(Type::DATE32, Date32Type)
     NUMERIC_CONVERTER_CASE(Type::DATE64, Date64Type)
+    NUMERIC_CONVERTER_CASE(Type::TIME32, Time32Type)
+    NUMERIC_CONVERTER_CASE(Type::TIME64, Time64Type)
+    NUMERIC_CONVERTER_CASE(Type::DURATION, DurationType)
     CONVERTER_CASE(Type::BOOL, (PrimitiveConverter<BooleanType, BooleanValueDecoder>))
     CONVERTER_CASE(Type::BINARY,
                    (PrimitiveConverter<BinaryType, BinaryValueDecoder<false>>))
@@ -576,8 +736,6 @@ Result<std::shared_ptr<Converter>> Converter::Make(const std::shared_ptr<DataTyp
                    (PrimitiveConverter<LargeBinaryType, BinaryValueDecoder<false>>))
     CONVERTER_CASE(Type::FIXED_SIZE_BINARY,
                    (PrimitiveConverter<FixedSizeBinaryType, FixedSizeBinaryValueDecoder>))
-    CONVERTER_CASE(Type::DECIMAL,
-                   (PrimitiveConverter<Decimal128Type, DecimalValueDecoder>))
 
     case Type::TIMESTAMP:
       ptr = MakeTimestampConverter<PrimitiveConverter>(type, options, pool);
@@ -623,6 +781,7 @@ Result<std::shared_ptr<Converter>> Converter::Make(const std::shared_ptr<DataTyp
 
 #undef CONVERTER_CASE
 #undef NUMERIC_CONVERTER_CASE
+#undef REAL_CONVERTER_CASE
   }
   RETURN_NOT_OK(ptr->Initialize());
   return ptr;
@@ -640,6 +799,12 @@ Result<std::shared_ptr<DictionaryConverter>> DictionaryConverter::Make(
         new TypedDictionaryConverter<TYPE, VALUE_DECODER_TYPE>(type, options, pool)); \
     break;
 
+#define REAL_CONVERTER_CASE(TYPE_ID, TYPE_CLASS, DECODER)                              \
+  case TYPE_ID:                                                                        \
+    ptr = MakeRealConverter<DictionaryConverter, TypedDictionaryConverter, TYPE_CLASS, \
+                            DECODER>(type, options, pool);                             \
+    break;
+
     // XXX Are 32-bit types useful?
     CONVERTER_CASE(Type::INT32, Int32Type, NumericValueDecoder<Int32Type>)
     CONVERTER_CASE(Type::INT64, Int64Type, NumericValueDecoder<Int64Type>)
@@ -647,7 +812,8 @@ Result<std::shared_ptr<DictionaryConverter>> DictionaryConverter::Make(
     CONVERTER_CASE(Type::UINT64, UInt64Type, NumericValueDecoder<UInt64Type>)
     CONVERTER_CASE(Type::FLOAT, FloatType, NumericValueDecoder<FloatType>)
     CONVERTER_CASE(Type::DOUBLE, DoubleType, NumericValueDecoder<DoubleType>)
-    CONVERTER_CASE(Type::DECIMAL, Decimal128Type, DecimalValueDecoder)
+    CONVERTER_CASE(Type::DURATION, DurationType, DurationValueDecoder)
+    REAL_CONVERTER_CASE(Type::DECIMAL, Decimal128Type, DecimalValueDecoder)
     CONVERTER_CASE(Type::FIXED_SIZE_BINARY, FixedSizeBinaryType,
                    FixedSizeBinaryValueDecoder)
     CONVERTER_CASE(Type::BINARY, BinaryType, BinaryValueDecoder<false>)
@@ -683,6 +849,7 @@ Result<std::shared_ptr<DictionaryConverter>> DictionaryConverter::Make(
     }
 
 #undef CONVERTER_CASE
+#undef REAL_CONVERTER_CASE
   }
   RETURN_NOT_OK(ptr->Initialize());
   return ptr;
