@@ -22,17 +22,16 @@
 #include "arrow/ipc/writer.h"
 #include "arrow/record_batch.h"
 #include "arrow/result.h"
-#include "arrow/result_internal.h"
 #include "arrow/stl_allocator.h"
 #include "arrow/util/iterator.h"
-#include "arrow/util/logging.h"
+#include "arrow/util/logging_internal.h"
 #include "arrow/visit_data_inline.h"
 #include "arrow/visit_type_inline.h"
 
 #include <memory>
 
 #if defined(ARROW_HAVE_NEON) || defined(ARROW_HAVE_SSE4_2)
-#include <xsimd/xsimd.hpp>
+#  include <xsimd/xsimd.hpp>
 #endif
 
 namespace arrow {
@@ -106,7 +105,8 @@ int64_t CountQuotes(std::string_view s) {
 
 // Matching quote pair character length.
 constexpr int64_t kQuoteCount = 2;
-constexpr int64_t kQuoteDelimiterCount = kQuoteCount + /*end_char*/ 1;
+// Delimiter character length.
+constexpr int64_t kDelimiterCount = 1;
 
 // Interface for generating CSV data per column.
 // The intended usage is to iteratively call UpdateRowLengths for a column and
@@ -129,15 +129,15 @@ class ColumnPopulator {
     // threading overhead would not be justified.
     ctx.set_use_threads(false);
     if (data.type() && is_large_binary_like(data.type()->id())) {
-      ASSIGN_OR_RAISE(array_, compute::Cast(data, /*to_type=*/large_utf8(),
-                                            compute::CastOptions(), &ctx));
+      ARROW_ASSIGN_OR_RAISE(array_, compute::Cast(data, /*to_type=*/large_utf8(),
+                                                  compute::CastOptions(), &ctx));
     } else {
       auto casted = compute::Cast(data, /*to_type=*/utf8(), compute::CastOptions(), &ctx);
       if (casted.ok()) {
         array_ = std::move(casted).ValueOrDie();
       } else if (casted.status().IsCapacityError()) {
-        ASSIGN_OR_RAISE(array_, compute::Cast(data, /*to_type=*/large_utf8(),
-                                              compute::CastOptions(), &ctx));
+        ARROW_ASSIGN_OR_RAISE(array_, compute::Cast(data, /*to_type=*/large_utf8(),
+                                                    compute::CastOptions(), &ctx));
       } else {
         return casted.status();
       }
@@ -177,6 +177,34 @@ char* Escape(std::string_view s, char* out) {
   return out;
 }
 
+// Return the index of the first structural char in the input. A structural char
+// is a character that needs quoting and/or escaping.
+int64_t StopAtStructuralChar(const uint8_t* data, const int64_t buffer_size,
+                             const char delimiter) {
+  int64_t offset = 0;
+#if defined(ARROW_HAVE_SSE4_2) || defined(ARROW_HAVE_NEON)
+  // _mm_cmpistrc gives slightly better performance than the naive approach,
+  // probably doesn't deserve the effort
+  using simd_batch = xsimd::make_sized_batch_t<uint8_t, 16>;
+  while ((offset + 16) <= buffer_size) {
+    const auto v = simd_batch::load_unaligned(data + offset);
+    if (xsimd::any((v == '\n') | (v == '\r') | (v == '"') | (v == delimiter))) {
+      break;
+    }
+    offset += 16;
+  }
+#endif
+  while (offset < buffer_size) {
+    // error happened or remaining bytes to check
+    const char c = static_cast<char>(data[offset]);
+    if (c == '\n' || c == '\r' || c == '"' || c == delimiter) {
+      break;
+    }
+    ++offset;
+  }
+  return offset;
+}
+
 // Populator used for non-string/binary types, or when unquoted strings/binary types are
 // desired. It assumes the strings in the casted array do not require quoting or escaping.
 // This is enforced by setting reject_values_with_quotes to true, in which case a check
@@ -269,35 +297,18 @@ class UnquotedColumnPopulator : public ColumnPopulator {
     // scan the underlying string array buffer as a single big string
     const uint8_t* const data = array.raw_data() + array.value_offset(0);
     const int64_t buffer_size = array.total_values_length();
-    int64_t offset = 0;
-#if defined(ARROW_HAVE_SSE4_2) || defined(ARROW_HAVE_NEON)
-    // _mm_cmpistrc gives slightly better performance than the naive approach,
-    // probably doesn't deserve the effort
-    using simd_batch = xsimd::make_sized_batch_t<uint8_t, 16>;
-    while ((offset + 16) <= buffer_size) {
-      const auto v = simd_batch::load_unaligned(data + offset);
-      if (xsimd::any((v == '\n') | (v == '\r') | (v == '"') | (v == delimiter))) {
-        break;
-      }
-      offset += 16;
-    }
-#endif
-    while (offset < buffer_size) {
-      // error happened or remaining bytes to check
-      const char c = static_cast<char>(data[offset]);
-      if (c == '\n' || c == '\r' || c == '"' || c == delimiter) {
-        // extract the offending string from array per offset
-        const auto* offsets = array.raw_value_offsets();
-        const auto index =
-            std::upper_bound(offsets, offsets + array.length(), offset + offsets[0]) -
-            offsets;
-        DCHECK_GT(index, 0);
-        return Status::Invalid(
-            "CSV values may not contain structural characters if quoting style is "
-            "\"None\". See RFC4180. Invalid value: ",
-            array.GetView(index - 1));
-      }
-      ++offset;
+    if (int64_t offset = StopAtStructuralChar(data, buffer_size, delimiter);
+        offset != buffer_size) {
+      // extract the offending string from array per offset
+      const auto* offsets = array.raw_value_offsets();
+      const auto index =
+          std::upper_bound(offsets, offsets + array.length(), offset + offsets[0]) -
+          offsets;
+      DCHECK_GT(index, 0);
+      return Status::Invalid(
+          "CSV values may not contain structural characters if quoting style is "
+          "\"None\". See RFC4180. Invalid value: ",
+          array.GetView(index - 1));
     }
     return Status::OK();
   }
@@ -501,8 +512,8 @@ class CSVWriterImpl : public ipc::RecordBatchWriter {
       return Status::Invalid("Null string cannot contain quotes.");
     }
 
-    ASSIGN_OR_RAISE(std::shared_ptr<Buffer> null_string,
-                    arrow::AllocateBuffer(options.null_string.length()));
+    ARROW_ASSIGN_OR_RAISE(std::shared_ptr<Buffer> null_string,
+                          arrow::AllocateBuffer(options.null_string.length()));
     memcpy(null_string->mutable_data(), options.null_string.data(),
            options.null_string.length());
 
@@ -511,7 +522,7 @@ class CSVWriterImpl : public ipc::RecordBatchWriter {
     for (int col = 0; col < schema->num_fields(); col++) {
       const std::string& end_chars =
           col < schema->num_fields() - 1 ? delimiter : options.eol;
-      ASSIGN_OR_RAISE(
+      ARROW_ASSIGN_OR_RAISE(
           populators[col],
           MakePopulator(*schema->field(col), end_chars, options.delimiter, null_string,
                         options.quoting_style, options.io_context.pool()));
@@ -528,7 +539,7 @@ class CSVWriterImpl : public ipc::RecordBatchWriter {
   Status WriteRecordBatch(const RecordBatch& batch) override {
     RecordBatchIterator iterator = RecordBatchSliceIterator(batch, options_.batch_size);
     for (auto maybe_slice : iterator) {
-      ASSIGN_OR_RAISE(std::shared_ptr<RecordBatch> slice, maybe_slice);
+      ARROW_ASSIGN_OR_RAISE(std::shared_ptr<RecordBatch> slice, maybe_slice);
       RETURN_NOT_OK(TranslateMinimalBatch(*slice));
       RETURN_NOT_OK(sink_->Write(data_buffer_));
       stats_.num_record_batches++;
@@ -570,34 +581,71 @@ class CSVWriterImpl : public ipc::RecordBatchWriter {
   Status PrepareForContentsWrite() {
     // Only called once, as part of initialization
     if (data_buffer_ == nullptr) {
-      ASSIGN_OR_RAISE(data_buffer_,
-                      AllocateResizableBuffer(
-                          options_.batch_size * schema_->num_fields() * kColumnSizeGuess,
-                          options_.io_context.pool()));
+      ARROW_ASSIGN_OR_RAISE(
+          data_buffer_,
+          AllocateResizableBuffer(
+              options_.batch_size * schema_->num_fields() * kColumnSizeGuess,
+              options_.io_context.pool()));
     }
     return Status::OK();
   }
 
-  int64_t CalculateHeaderSize() const {
+  int64_t CalculateHeaderSize(QuotingStyle quoting_style) const {
     int64_t header_length = 0;
     for (int col = 0; col < schema_->num_fields(); col++) {
       const std::string& col_name = schema_->field(col)->name();
       header_length += col_name.size();
-      header_length += CountQuotes(col_name);
+      switch (quoting_style) {
+        case QuotingStyle::None:
+          break;
+        case QuotingStyle::Needed:
+        case QuotingStyle::AllValid:
+          header_length += CountQuotes(col_name);
+          break;
+      }
     }
-    // header_length + ([quotes + ','] * schema_->num_fields()) + (eol - ',')
-    return header_length + (kQuoteDelimiterCount * schema_->num_fields()) +
-           (options_.eol.size() - 1);
+    header_length += kDelimiterCount * (schema_->num_fields() - 1) + options_.eol.size();
+    switch (quoting_style) {
+      case QuotingStyle::None:
+        break;
+      case QuotingStyle::Needed:
+      case QuotingStyle::AllValid:
+        header_length += kQuoteCount * schema_->num_fields();
+        break;
+    }
+    return header_length;
   }
 
   Status WriteHeader() {
     // Only called once, as part of initialization
-    RETURN_NOT_OK(data_buffer_->Resize(CalculateHeaderSize(), /*shrink_to_fit=*/false));
+    RETURN_NOT_OK(data_buffer_->Resize(CalculateHeaderSize(options_.quoting_header),
+                                       /*shrink_to_fit=*/false));
     char* next = reinterpret_cast<char*>(data_buffer_->mutable_data());
     for (int col = 0; col < schema_->num_fields(); ++col) {
-      *next++ = '"';
-      next = Escape(schema_->field(col)->name(), next);
-      *next++ = '"';
+      const std::string& col_name = schema_->field(col)->name();
+      switch (options_.quoting_header) {
+        case QuotingStyle::None:
+          if (StopAtStructuralChar(reinterpret_cast<const uint8_t*>(col_name.c_str()),
+                                   col_name.length(), options_.delimiter) !=
+              static_cast<int64_t>(col_name.length())) {
+            return Status::Invalid(
+                "CSV header may not contain structural characters if quoting style is "
+                "\"None\". See RFC4180. Invalid value: ",
+                col_name);
+          }
+          memcpy(next, col_name.data(), col_name.size());
+          next += col_name.size();
+          break;
+        case QuotingStyle::Needed:
+        case QuotingStyle::AllValid:
+          // QuotingStyle::Needed is defined as always quoting string/binary data,
+          // regardless of whether it contains structural chars.
+          // We use consistent semantics for header names, which are strings.
+          *next++ = '"';
+          next = Escape(schema_->field(col)->name(), next);
+          *next++ = '"';
+          break;
+      }
       if (col != schema_->num_fields() - 1) {
         *next++ = options_.delimiter;
       }
@@ -665,24 +713,24 @@ class CSVWriterImpl : public ipc::RecordBatchWriter {
 
 Status WriteCSV(const Table& table, const WriteOptions& options,
                 arrow::io::OutputStream* output) {
-  ASSIGN_OR_RAISE(auto writer, MakeCSVWriter(output, table.schema(), options));
+  ARROW_ASSIGN_OR_RAISE(auto writer, MakeCSVWriter(output, table.schema(), options));
   RETURN_NOT_OK(writer->WriteTable(table));
   return writer->Close();
 }
 
 Status WriteCSV(const RecordBatch& batch, const WriteOptions& options,
                 arrow::io::OutputStream* output) {
-  ASSIGN_OR_RAISE(auto writer, MakeCSVWriter(output, batch.schema(), options));
+  ARROW_ASSIGN_OR_RAISE(auto writer, MakeCSVWriter(output, batch.schema(), options));
   RETURN_NOT_OK(writer->WriteRecordBatch(batch));
   return writer->Close();
 }
 
 Status WriteCSV(const std::shared_ptr<RecordBatchReader>& reader,
                 const WriteOptions& options, arrow::io::OutputStream* output) {
-  ASSIGN_OR_RAISE(auto writer, MakeCSVWriter(output, reader->schema(), options));
+  ARROW_ASSIGN_OR_RAISE(auto writer, MakeCSVWriter(output, reader->schema(), options));
   std::shared_ptr<RecordBatch> batch;
   while (true) {
-    ASSIGN_OR_RAISE(batch, reader->Next());
+    ARROW_ASSIGN_OR_RAISE(batch, reader->Next());
     if (batch == nullptr) break;
     RETURN_NOT_OK(writer->WriteRecordBatch(*batch));
   }
