@@ -53,6 +53,10 @@ using internal::SerialExecutor;
 
 namespace dataset {
 
+namespace {
+
+using RecordBatchGenerator = std::function<Future<std::shared_ptr<RecordBatch>>()>;
+
 struct CsvInspectedFragment : public InspectedFragment {
   CsvInspectedFragment(std::vector<std::string> column_names,
                        std::shared_ptr<io::InputStream> input_stream, int64_t num_bytes)
@@ -142,8 +146,6 @@ class CsvFileScanner : public FragmentScanner {
   int scanned_so_far_ = 0;
 };
 
-using RecordBatchGenerator = std::function<Future<std::shared_ptr<RecordBatch>>()>;
-
 Result<std::vector<std::string>> GetOrderedColumnNames(
     const csv::ReadOptions& read_options, const csv::ParseOptions& parse_options,
     std::string_view first_block, MemoryPool* pool) {
@@ -348,6 +350,8 @@ static RecordBatchGenerator GeneratorFromReader(
   return MakeFromFuture(std::move(gen_fut));
 }
 
+}  // namespace
+
 CsvFileFormat::CsvFileFormat() : FileFormat(std::make_shared<CsvFragmentScanOptions>()) {}
 
 bool CsvFileFormat::Equals(const FileFormat& format) const {
@@ -420,6 +424,8 @@ Future<std::shared_ptr<FragmentScanner>> CsvFileFormat::BeginScan(
                               exec_context->executor());
 }
 
+namespace {
+
 Result<std::shared_ptr<InspectedFragment>> DoInspectFragment(
     const FileSource& source, const CsvFragmentScanOptions& csv_options,
     compute::ExecContext* exec_context) {
@@ -442,6 +448,8 @@ Result<std::shared_ptr<InspectedFragment>> DoInspectFragment(
                                                 source.Size());
 }
 
+}  // namespace
+
 Future<std::shared_ptr<InspectedFragment>> CsvFileFormat::InspectFragment(
     const FileSource& source, const FragmentScanOptions* format_options,
     compute::ExecContext* exec_context) const {
