@@ -17,7 +17,9 @@
 
 #include "odbcabstraction/calendar_utils.h"
 
+#include <chrono>
 #include <cstdint>
+#include <cstring>
 #include <ctime>
 
 namespace driver {
@@ -26,7 +28,7 @@ int64_t GetTodayTimeFromEpoch() {
   tm date{};
   int64_t t = std::time(0);
 
-  GetTimeForSecondsSinceEpoch(date, t);
+  GetTimeForSecondsSinceEpoch(t, date);
 
   date.tm_hour = 0;
   date.tm_min = 0;
@@ -39,13 +41,22 @@ int64_t GetTodayTimeFromEpoch() {
 #endif
 }
 
-void GetTimeForSecondsSinceEpoch(tm& date, int64_t value) {
-#if defined(_WIN32)
-  gmtime_s(&date, &value);
-#else
-  time_t time_value = static_cast<time_t>(value);
-  gmtime_r(&time_value, &date);
-#endif
+void GetTimeForSecondsSinceEpoch(const int64_t seconds_since_epoch, std::tm& out_tm) {
+  std::memset(&out_tm, 0, sizeof(std::tm));
+
+  std::chrono::time_point<std::chrono::system_clock, std::chrono::seconds> timepoint{
+      std::chrono::seconds{seconds_since_epoch}};
+  auto tm_days = std::chrono::floor<std::chrono::days>(timepoint);
+
+  std::chrono::year_month_day ymd(tm_days);
+  std::chrono::hh_mm_ss<std::chrono::seconds> timeofday(timepoint - tm_days);
+
+  out_tm.tm_year = static_cast<int>(ymd.year()) - 1900;
+  out_tm.tm_mon = static_cast<unsigned>(ymd.month()) - 1;
+  out_tm.tm_mday = static_cast<unsigned>(ymd.day());
+  out_tm.tm_hour = static_cast<int>(timeofday.hours().count());
+  out_tm.tm_min = static_cast<int>(timeofday.minutes().count());
+  out_tm.tm_sec = static_cast<int>(timeofday.seconds().count());
 }
 }  // namespace odbcabstraction
 }  // namespace driver
