@@ -1194,6 +1194,8 @@ void IpcOptionsTest::TestDoExchangeServerWriteOptions() {
 
 #if defined(ARROW_CUDA)
 
+namespace {
+
 Status CheckBuffersOnDevice(const Array& array, const Device& device) {
   if (array.num_fields() != 0) {
     return Status::NotImplemented("Nested arrays");
@@ -1284,6 +1286,8 @@ class CudaTestServer : public FlightServerBase {
   std::shared_ptr<cuda::CudaContext> context_;
 };
 
+}  // namespace
+
 // Store CUDA objects without exposing them in the public header
 class CudaDataTest::Impl {
  public:
@@ -1750,7 +1754,7 @@ void ErrorHandlingTest::TestGetFlightInfoMetadata() {
                             }));
 }
 
-void CheckErrorDetail(const Status& status) {
+static void CheckErrorDetail(const Status& status) {
   auto detail = FlightStatusDetail::UnwrapStatus(status);
   ASSERT_NE(detail, nullptr) << status.ToString();
   ASSERT_EQ(detail->code(), FlightStatusCode::Unauthorized);
