@@ -16,8 +16,8 @@
 // under the License.
 
 #ifndef _WIN32
-#include <fcntl.h>  // IWYU pragma: keep
-#include <unistd.h>
+#  include <fcntl.h>  // IWYU pragma: keep
+#  include <unistd.h>
 #endif
 
 #include <atomic>
@@ -36,11 +36,13 @@
 #include "arrow/buffer.h"
 #include "arrow/io/file.h"
 #include "arrow/io/interfaces.h"
+#include "arrow/io/stdio.h"
 #include "arrow/io/test_common.h"
 #include "arrow/memory_pool.h"
 #include "arrow/status.h"
 #include "arrow/testing/gtest_util.h"
 #include "arrow/testing/util.h"
+#include "arrow/util/config.h"
 #include "arrow/util/future.h"
 #include "arrow/util/io_util.h"
 
@@ -387,6 +389,22 @@ TEST_F(TestReadableFile, ReadAsync) {
   AssertBufferEqual(*buf2, "test");
 }
 
+TEST_F(TestReadableFile, ReadManyAsync) {
+  MakeTestFile();
+  OpenFile();
+
+  std::vector<ReadRange> ranges = {{1, 3}, {2, 5}, {4, 2}};
+  auto futs = file_->ReadManyAsync(std::move(ranges));
+
+  ASSERT_EQ(futs.size(), 3);
+  ASSERT_OK_AND_ASSIGN(auto buf1, futs[0].result());
+  ASSERT_OK_AND_ASSIGN(auto buf2, futs[1].result());
+  ASSERT_OK_AND_ASSIGN(auto buf3, futs[2].result());
+  AssertBufferEqual(*buf1, "est");
+  AssertBufferEqual(*buf2, "stdat");
+  AssertBufferEqual(*buf3, "da");
+}
+
 TEST_F(TestReadableFile, SeekingRequired) {
   MakeTestFile();
   OpenFile();
@@ -416,22 +434,25 @@ TEST_F(TestReadableFile, NonexistentFile) {
   auto maybe_file = ReadableFile::Open(path);
   ASSERT_RAISES(IOError, maybe_file);
   std::string message = maybe_file.status().message();
-  ASSERT_NE(std::string::npos, message.find(path));
+  ASSERT_NE(std::string::npos, message.find(path)) << message;
 }
 
 class MyMemoryPool : public MemoryPool {
  public:
   MyMemoryPool() : num_allocations_(0) {}
 
-  Status Allocate(int64_t size, uint8_t** out) override {
+  Status Allocate(int64_t size, int64_t /*alignment*/, uint8_t** out) override {
     *out = reinterpret_cast<uint8_t*>(std::malloc(size));
     ++num_allocations_;
     return Status::OK();
   }
 
-  void Free(uint8_t* buffer, int64_t size) override { std::free(buffer); }
+  void Free(uint8_t* buffer, int64_t size, int64_t /*alignment*/) override {
+    std::free(buffer);
+  }
 
-  Status Reallocate(int64_t old_size, int64_t new_size, uint8_t** ptr) override {
+  Status Reallocate(int64_t old_size, int64_t new_size, int64_t /*alignment*/,
+                    uint8_t** ptr) override {
     *ptr = reinterpret_cast<uint8_t*>(std::realloc(*ptr, new_size));
 
     if (*ptr == NULL) {
@@ -443,9 +464,11 @@ class MyMemoryPool : public MemoryPool {
 
   int64_t bytes_allocated() const override { return -1; }
 
+  int64_t total_bytes_allocated() const override { return -1; }
+
   std::string backend_name() const override { return "my"; }
 
-  int64_t num_allocations() const { return num_allocations_.load(); }
+  int64_t num_allocations() const override { return num_allocations_.load(); }
 
  private:
   std::atomic<int64_t> num_allocations_;
@@ -464,6 +487,10 @@ TEST_F(TestReadableFile, CustomMemoryPool) {
 }
 
 TEST_F(TestReadableFile, ThreadSafety) {
+#ifndef ARROW_ENABLE_THREADING
+  GTEST_SKIP() << "Test requires threading support";
+#endif
+
   std::string data = "foobar";
   {
     std::ofstream stream;
@@ -518,6 +545,9 @@ class TestPipeIO : public ::testing::Test {
 };
 
 TEST_F(TestPipeIO, TestWrite) {
+#ifdef __EMSCRIPTEN__
+  GTEST_SKIP() << "Pipes not supported on Emscripten";
+#endif
   std::string data1 = "test", data2 = "data!";
   std::shared_ptr<FileOutputStream> file;
   uint8_t buffer[10];
@@ -548,6 +578,9 @@ TEST_F(TestPipeIO, TestWrite) {
 }
 
 TEST_F(TestPipeIO, ReadableFileFails) {
+#ifdef __EMSCRIPTEN__
+  GTEST_SKIP() << "Pipes not supported on Emscripten";
+#endif
   // ReadableFile fails on non-seekable fd
   ASSERT_RAISES(IOError, ReadableFile::Open(pipe_.rfd.fd()));
 }
@@ -1026,6 +1059,10 @@ TEST_F(TestMemoryMappedFile, CastableToFileInterface) {
 }
 
 TEST_F(TestMemoryMappedFile, ThreadSafety) {
+#ifndef ARROW_ENABLE_THREADING
+  GTEST_SKIP() << "Test requires threading support";
+#endif
+
   std::string data = "foobar";
   std::string path = TempFile("ipc-multithreading-test");
   CreateFile(path, static_cast<int>(data.size()));
@@ -1054,5 +1091,64 @@ TEST_F(TestMemoryMappedFile, ThreadSafety) {
   ASSERT_EQ(niter * 2, correct_count);
 }
 
+// ----------------------------------------------------------------------
+// Stdio tests
+
+class TestStdio : public FileTestFixture {
+ public:
+  void CreateStdinWithData(const char* data, size_t size) {
+    EnsureFileDeleted();
+
+    ASSERT_OK_AND_ASSIGN(auto file, FileOutputStream::Open(path_, false));
+    ASSERT_OK(file->Write(data, size));
+    ASSERT_OK(file->Close());
+    cin_.reset(new std::ifstream(path_));
+    std::cin.rdbuf(cin_->rdbuf());
+  }
+
+ protected:
+  std::shared_ptr<std::ifstream> cin_;
+};
+
+TEST_F(TestStdio, ReadStdinReadAtOnce) {
+  const char data[] = "testdata";
+  CreateStdinWithData(data, sizeof(data));
+
+  StdinStream input;
+  char buffer[sizeof(data)];
+  ASSERT_OK_AND_ASSIGN(auto res, input.Read(sizeof(buffer), buffer));
+  ASSERT_EQ(sizeof(data), res);
+  ASSERT_EQ(0, std::memcmp(buffer, data, sizeof(data)));
+  ASSERT_EQ(sizeof(data), input.Tell());
+}
+
+TEST_F(TestStdio, ReadStdinReadUnalignedBuffer) {
+  const char data[] = "testdata";
+  CreateStdinWithData(data, sizeof(data));
+
+  StdinStream input;
+  char buffer[sizeof(data) + 16];
+  ASSERT_OK_AND_ASSIGN(auto res, input.Read(sizeof(buffer), buffer));
+  ASSERT_EQ(sizeof(data), res);
+  ASSERT_EQ(0, std::memcmp(buffer, data, sizeof(data)));
+  ASSERT_EQ(sizeof(data), input.Tell());
+}
+
+TEST_F(TestStdio, ReadStdinReadAfterClose) {
+  const char data[] = "testdata";
+  CreateStdinWithData(data, sizeof(data));
+
+  StdinStream input;
+  char buffer[4];
+  ASSERT_OK_AND_ASSIGN(auto res, input.Read(sizeof(buffer), buffer));
+  ASSERT_EQ(sizeof(buffer), res);
+  ASSERT_EQ(0, std::memcmp(buffer, data, sizeof(buffer)));
+  ASSERT_EQ(sizeof(buffer), input.Tell());
+  cin_->close();
+  ASSERT_OK_AND_ASSIGN(res, input.Read(sizeof(buffer), buffer));
+  ASSERT_EQ(0, res);
+  ASSERT_EQ(sizeof(buffer), input.Tell());
+}
+
 }  // namespace io
 }  // namespace arrow
