@@ -16,10 +16,11 @@
 // under the License.
 
 #include <gtest/gtest.h>
-#include <time.h>
 
-#include "../execution_context.h"
+#include "arrow/util/logging_internal.h"
+#include "gandiva/execution_context.h"
 #include "gandiva/precompiled/testing.h"
+#include "gandiva/precompiled/time_constants.h"
 #include "gandiva/precompiled/types.h"
 
 namespace gandiva {
@@ -132,6 +133,48 @@ TEST(TestTime, TestCastTimestamp) {
   context.Reset();
 }
 
+TEST(TestTime, TestCastTimeUtf8) {
+  ExecutionContext context;
+  auto context_ptr = reinterpret_cast<int64_t>(&context);
+
+  EXPECT_EQ(castTIME_utf8(context_ptr, "9:45:30", 7), 35130000);
+  EXPECT_EQ(castTIME_utf8(context_ptr, "9:45:30.920", 11), 35130920);
+
+  EXPECT_EQ(castTIME_utf8(context_ptr, "9:45:30.1", 9),
+            castTIME_utf8(context_ptr, "9:45:30", 7) + 100);
+
+  EXPECT_EQ(castTIME_utf8(context_ptr, "9:45:30.10", 10),
+            castTIME_utf8(context_ptr, "9:45:30", 7) + 100);
+
+  EXPECT_EQ(castTIME_utf8(context_ptr, "9:45:30.100", 11),
+            castTIME_utf8(context_ptr, "9:45:30", 7) + 100);
+
+  // error cases
+  EXPECT_EQ(castTIME_utf8(context_ptr, "24H00H00", 8), 0);
+  EXPECT_EQ(context.get_error(), "Invalid character in time 24H00H00");
+  context.Reset();
+
+  EXPECT_EQ(castTIME_utf8(context_ptr, "24:00:00", 8), 0);
+  EXPECT_EQ(context.get_error(), "Not a valid time value 24:00:00");
+  context.Reset();
+
+  EXPECT_EQ(castTIME_utf8(context_ptr, "00:60:00", 8), 0);
+  EXPECT_EQ(context.get_error(), "Not a valid time value 00:60:00");
+  context.Reset();
+
+  EXPECT_EQ(castTIME_utf8(context_ptr, "00:00:100", 9), 0);
+  EXPECT_EQ(context.get_error(), "Not a valid time value 00:00:100");
+  context.Reset();
+
+  EXPECT_EQ(castTIME_utf8(context_ptr, "00:00:00.0001", 13), 0);
+  EXPECT_EQ(context.get_error(), "Invalid millis for time value 00:00:00.0001");
+  context.Reset();
+
+  EXPECT_EQ(castTIME_utf8(context_ptr, "00:00:00.1000", 13), 0);
+  EXPECT_EQ(context.get_error(), "Invalid millis for time value 00:00:00.1000");
+  context.Reset();
+}
+
 #ifndef _WIN32
 
 // TODO(wesm): ARROW-4495. Need to address TZ database issues on Windows
@@ -181,6 +224,40 @@ TEST(TestTime, TestExtractTime) {
   EXPECT_EQ(extractSecond_time32(time_as_millis_in_day), 33);
 }
 
+TEST(TestTime, TestDateDiff) {
+  gdv_timestamp ts1 = StringToTimestamp("2019-06-30 00:00:00");
+  gdv_timestamp ts2 = StringToTimestamp("2019-05-31 00:00:00");
+  EXPECT_EQ(datediff_timestamp_timestamp(ts1, ts2), 30);
+
+  ts1 = StringToTimestamp("2019-06-30 00:00:00");
+  ts2 = StringToTimestamp("2019-02-28 00:00:00");
+  EXPECT_EQ(datediff_timestamp_timestamp(ts1, ts2), 122);
+
+  ts1 = StringToTimestamp("2019-06-30 00:00:00");
+  ts2 = StringToTimestamp("2019-03-31 00:00:00");
+  EXPECT_EQ(datediff_timestamp_timestamp(ts1, ts2), 91);
+
+  ts1 = StringToTimestamp("2019-06-30 00:00:00");
+  ts2 = StringToTimestamp("2019-06-30 00:00:00");
+  EXPECT_EQ(datediff_timestamp_timestamp(ts1, ts2), 0);
+
+  ts1 = StringToTimestamp("2019-06-30 00:00:00");
+  ts2 = StringToTimestamp("2019-07-01 00:00:00");
+  EXPECT_EQ(datediff_timestamp_timestamp(ts1, ts2), -1);
+
+  ts1 = StringToTimestamp("2019-06-30 00:00:00");
+  ts2 = StringToTimestamp("2019-07-31 00:00:00");
+  EXPECT_EQ(datediff_timestamp_timestamp(ts1, ts2), -31);
+
+  ts1 = StringToTimestamp("2019-06-30 00:00:00");
+  ts2 = StringToTimestamp("2019-07-30 00:00:00");
+  EXPECT_EQ(datediff_timestamp_timestamp(ts1, ts2), -30);
+
+  ts1 = StringToTimestamp("2019-06-30 00:00:00");
+  ts2 = StringToTimestamp("2019-07-29 00:00:00");
+  EXPECT_EQ(datediff_timestamp_timestamp(ts1, ts2), -29);
+}
+
 TEST(TestTime, TestTimestampDiffMonth) {
   gdv_timestamp ts1 = StringToTimestamp("2019-06-30 00:00:00");
   gdv_timestamp ts2 = StringToTimestamp("2019-05-31 00:00:00");
@@ -286,6 +363,46 @@ TEST(TestTime, TimeStampTrunc) {
             StringToTimestamp("2000-02-28 00:00:00"));
   EXPECT_EQ(date_trunc_Week_timestamp(StringToTimestamp("2000-03-06 10:10:10")),
             StringToTimestamp("2000-03-06 00:00:00"));
+
+  // Test dates before epoch (1970-01-01)
+  EXPECT_EQ(date_trunc_Second_date64(StringToTimestamp("1969-12-31 23:45:15")),
+            StringToTimestamp("1969-12-31 23:45:15"));
+  EXPECT_EQ(date_trunc_Minute_date64(StringToTimestamp("1969-12-31 23:45:15")),
+            StringToTimestamp("1969-12-31 23:45:00"));
+  EXPECT_EQ(date_trunc_Hour_date64(StringToTimestamp("1969-12-31 23:45:15")),
+            StringToTimestamp("1969-12-31 23:00:00"));
+  EXPECT_EQ(date_trunc_Day_date64(StringToTimestamp("1969-12-31 23:45:15")),
+            StringToTimestamp("1969-12-31 00:00:00"));
+  EXPECT_EQ(date_trunc_Month_date64(StringToTimestamp("1969-12-31 23:45:15")),
+            StringToTimestamp("1969-12-01 00:00:00"));
+  EXPECT_EQ(date_trunc_Quarter_date64(StringToTimestamp("1969-12-31 23:45:15")),
+            StringToTimestamp("1969-10-01 00:00:00"));
+  EXPECT_EQ(date_trunc_Year_date64(StringToTimestamp("1969-12-31 23:45:15")),
+            StringToTimestamp("1969-01-01 00:00:00"));
+  EXPECT_EQ(date_trunc_Decade_date64(StringToTimestamp("1969-12-31 23:45:15")),
+            StringToTimestamp("1960-01-01 00:00:00"));
+
+  // Test date further in the past
+  EXPECT_EQ(date_trunc_Second_date64(StringToTimestamp("1930-05-15 12:34:56")),
+            StringToTimestamp("1930-05-15 12:34:56"));
+  EXPECT_EQ(date_trunc_Minute_date64(StringToTimestamp("1930-05-15 12:34:56")),
+            StringToTimestamp("1930-05-15 12:34:00"));
+  EXPECT_EQ(date_trunc_Hour_date64(StringToTimestamp("1930-05-15 12:34:56")),
+            StringToTimestamp("1930-05-15 12:00:00"));
+  EXPECT_EQ(date_trunc_Day_date64(StringToTimestamp("1930-05-15 12:34:56")),
+            StringToTimestamp("1930-05-15 00:00:00"));
+  EXPECT_EQ(date_trunc_Day_date64(StringToTimestamp("1940-02-29 12:00:00")),
+            StringToTimestamp("1940-02-29 00:00:00"));
+  EXPECT_EQ(date_trunc_Month_date64(StringToTimestamp("1930-05-15 12:34:56")),
+            StringToTimestamp("1930-05-01 00:00:00"));
+  EXPECT_EQ(date_trunc_Quarter_date64(StringToTimestamp("1930-05-15 12:34:56")),
+            StringToTimestamp("1930-04-01 00:00:00"));
+  EXPECT_EQ(date_trunc_Year_date64(StringToTimestamp("1930-05-15 12:34:56")),
+            StringToTimestamp("1930-01-01 00:00:00"));
+  EXPECT_EQ(date_trunc_Decade_date64(StringToTimestamp("1931-05-15 12:34:56")),
+            StringToTimestamp("1930-01-01 00:00:00"));
+  EXPECT_EQ(date_trunc_Century_date64(StringToTimestamp("1930-05-15 12:34:56")),
+            StringToTimestamp("1901-01-01 00:00:00"));
 }
 
 TEST(TestTime, TimeStampAdd) {
@@ -293,25 +410,62 @@ TEST(TestTime, TimeStampAdd) {
       timestampaddSecond_int32_timestamp(30, StringToTimestamp("2000-05-01 10:20:34")),
       StringToTimestamp("2000-05-01 10:21:04"));
 
+  EXPECT_EQ(
+      timestampaddSecond_timestamp_int32(StringToTimestamp("2000-05-01 10:20:34"), 30),
+      StringToTimestamp("2000-05-01 10:21:04"));
+
   EXPECT_EQ(
       timestampaddMinute_int64_timestamp(-30, StringToTimestamp("2000-05-01 10:20:34")),
       StringToTimestamp("2000-05-01 09:50:34"));
 
+  EXPECT_EQ(
+      timestampaddMinute_timestamp_int64(StringToTimestamp("2000-05-01 10:20:34"), -30),
+      StringToTimestamp("2000-05-01 09:50:34"));
+
   EXPECT_EQ(
       timestampaddHour_int32_timestamp(20, StringToTimestamp("2000-05-01 10:20:34")),
       StringToTimestamp("2000-05-02 06:20:34"));
 
+  EXPECT_EQ(
+      timestampaddHour_timestamp_int32(StringToTimestamp("2000-05-01 10:20:34"), 20),
+      StringToTimestamp("2000-05-02 06:20:34"));
+
   EXPECT_EQ(
       timestampaddDay_int64_timestamp(-35, StringToTimestamp("2000-05-01 10:20:34")),
       StringToTimestamp("2000-03-27 10:20:34"));
 
+  EXPECT_EQ(
+      timestampaddDay_timestamp_int64(StringToTimestamp("2000-05-01 10:20:34"), -35),
+      StringToTimestamp("2000-03-27 10:20:34"));
+
   EXPECT_EQ(timestampaddWeek_int32_timestamp(4, StringToTimestamp("2000-05-01 10:20:34")),
             StringToTimestamp("2000-05-29 10:20:34"));
 
+  EXPECT_EQ(timestampaddWeek_timestamp_int32(StringToTimestamp("2000-05-01 10:20:34"), 4),
+            StringToTimestamp("2000-05-29 10:20:34"));
+
+  EXPECT_EQ(timestampaddWeek_timestamp_int32(StringToTimestamp("2000-05-01 10:20:34"), 4),
+            StringToTimestamp("2000-05-29 10:20:34"));
+
   EXPECT_EQ(
       timestampaddMonth_int64_timestamp(10, StringToTimestamp("2000-05-01 10:20:34")),
       StringToTimestamp("2001-03-01 10:20:34"));
 
+  EXPECT_EQ(
+      timestampaddMonth_int64_timestamp(1, StringToTimestamp("2000-01-31 10:20:34")),
+      StringToTimestamp("2000-2-29 10:20:34"));
+  EXPECT_EQ(
+      timestampaddMonth_int64_timestamp(13, StringToTimestamp("2001-01-31 10:20:34")),
+      StringToTimestamp("2002-02-28 10:20:34"));
+
+  EXPECT_EQ(
+      timestampaddMonth_int64_timestamp(11, StringToTimestamp("2000-05-31 10:20:34")),
+      StringToTimestamp("2001-04-30 10:20:34"));
+
+  EXPECT_EQ(
+      timestampaddMonth_timestamp_int64(StringToTimestamp("2000-05-31 10:20:34"), 11),
+      StringToTimestamp("2001-04-30 10:20:34"));
+
   EXPECT_EQ(
       timestampaddQuarter_int32_timestamp(-2, StringToTimestamp("2000-05-01 10:20:34")),
       StringToTimestamp("1999-11-01 10:20:34"));
@@ -674,6 +828,11 @@ TEST(TestTime, TestExtractWeek) {
   }
 }
 
+TEST(TestTime, TestIsNullInterval) {
+  EXPECT_EQ(isnull_day_time_interval(150, true), false);
+  EXPECT_EQ(isnull_day_time_interval(150, false), true);
+}
+
 TEST(TestTime, TestMonthsBetween) {
   std::vector<std::string> testStrings = {
       "1995-03-02 00:00:00", "1995-02-02 00:00:00", "1.0",
@@ -729,6 +888,42 @@ TEST(TestTime, TestCastTimestampToDate) {
   EXPECT_EQ(StringToTimestamp("2000-05-01 00:00:00"), out);
 }
 
+TEST(TestTime, TestNextDay) {
+  ExecutionContext context;
+  int64_t context_ptr = reinterpret_cast<int64_t>(&context);
+
+  gdv_timestamp ts = StringToTimestamp("2021-11-08 10:20:34");
+  auto out = next_day_from_timestamp(context_ptr, ts, "FR", 2);
+  EXPECT_EQ(StringToTimestamp("2021-11-12 00:00:00"), out);
+
+  out = next_day_from_timestamp(context_ptr, ts, "FRI", 3);
+  EXPECT_EQ(StringToTimestamp("2021-11-12 00:00:00"), out);
+
+  out = next_day_from_timestamp(context_ptr, ts, "FRIDAY", 6);
+  EXPECT_EQ(StringToTimestamp("2021-11-12 00:00:00"), out);
+
+  ts = StringToTimestamp("2015-08-06 11:12:30");
+  out = next_day_from_timestamp(context_ptr, ts, "THU", 3);
+  EXPECT_EQ(StringToTimestamp("2015-08-13 00:00:00"), out);
+
+  ts = StringToTimestamp("2012-08-14 11:12:30");
+  out = next_day_from_timestamp(context_ptr, ts, "TUE", 3);
+  EXPECT_EQ(StringToTimestamp("2012-08-21 00:00:00"), out);
+
+  ts = StringToTimestamp("2012-12-12 12:00:00");
+  out = next_day_from_timestamp(context_ptr, ts, "TU", 2);
+  EXPECT_EQ(StringToTimestamp("2012-12-18 00:00:00"), out);
+
+  ts = StringToTimestamp("2000-01-01 20:15:00");
+  out = next_day_from_timestamp(context_ptr, ts, "SATURDAY", 8);
+  EXPECT_EQ(StringToTimestamp("2000-01-08 00:00:00"), out);
+
+  ts = StringToTimestamp("2015-08-06 11:12:30");
+  out = next_day_from_timestamp(context_ptr, ts, "AHSRK", 5);
+  EXPECT_EQ(context.get_error(), "The weekday in this entry is invalid");
+  context.Reset();
+}
+
 TEST(TestTime, TestCastTimestampToTime) {
   gdv_timestamp ts = StringToTimestamp("2000-05-01 10:20:34");
   auto expected_response =
@@ -748,6 +943,28 @@ TEST(TestTime, TestCastTimestampToTime) {
   EXPECT_EQ(expected_response, out);
 }
 
+TEST(TestTime, TestIntToTime) {
+  int32_t val = 1000;
+  int32_t expected_response = val;
+  auto out = castTIME_int32(val);
+  EXPECT_EQ(expected_response, out);
+
+  val = MILLIS_IN_DAY - 1;
+  expected_response = val;
+  out = castTIME_int32(val);
+  EXPECT_EQ(expected_response, out);
+
+  val = MILLIS_IN_DAY + 1;
+  expected_response = 1;
+  out = castTIME_int32(val);
+  EXPECT_EQ(expected_response, out);
+
+  val = -1;
+  expected_response = 0;
+  out = castTIME_int32(val);
+  EXPECT_EQ(expected_response, out);
+}
+
 TEST(TestTime, TestLastDay) {
   // leap year test
   gdv_timestamp ts = StringToTimestamp("2016-02-11 03:20:34");
@@ -786,6 +1003,10 @@ TEST(TestTime, TestToTimestamp) {
   EXPECT_EQ(ts, to_timestamp_float32(1));
   EXPECT_EQ(ts, to_timestamp_float64(1));
 
+  ts = StringToTimestamp("2021-07-14 09:31:39");
+  EXPECT_EQ(ts, to_timestamp_int32(1626255099));
+  EXPECT_EQ(ts, to_timestamp_int64(1626255099));
+
   ts = StringToTimestamp("1970-01-01 00:01:00");
   EXPECT_EQ(ts, to_timestamp_int32(60));
   EXPECT_EQ(ts, to_timestamp_int64(60));
@@ -869,4 +1090,48 @@ TEST(TestTime, TestToTimeNumeric) {
   EXPECT_EQ(expected_output, to_time_float64(3601.500));
 }
 
+TEST(TestTime, TestCastIntDayInterval) {
+  EXPECT_EQ(castBIGINT_daytimeinterval(10), 864000000);
+  EXPECT_EQ(castBIGINT_daytimeinterval(-100), -8640000001);
+  EXPECT_EQ(castBIGINT_daytimeinterval(-0), 0);
+}
+
+TEST(TestTime, TestCastIntYearInterval) {
+  EXPECT_EQ(castINT_year_interval(24), 2);
+  EXPECT_EQ(castINT_year_interval(-24), -2);
+  EXPECT_EQ(castINT_year_interval(-23), -1);
+
+  EXPECT_EQ(castBIGINT_year_interval(24), 2);
+  EXPECT_EQ(castBIGINT_year_interval(-24), -2);
+  EXPECT_EQ(castBIGINT_year_interval(-23), -1);
+}
+
+TEST(TestTime, TestCastNullableInterval) {
+  ExecutionContext context;
+  auto context_ptr = reinterpret_cast<int64_t>(&context);
+  // Test castNULLABLEINTERVALDAY for int and bigint
+  EXPECT_EQ(castNULLABLEINTERVALDAY_int32(1), 1);
+  EXPECT_EQ(castNULLABLEINTERVALDAY_int32(12), 12);
+  EXPECT_EQ(castNULLABLEINTERVALDAY_int32(-55), -55);
+  EXPECT_EQ(castNULLABLEINTERVALDAY_int32(-1201), -1201);
+  EXPECT_EQ(castNULLABLEINTERVALDAY_int64(1), 1);
+  EXPECT_EQ(castNULLABLEINTERVALDAY_int64(12), 12);
+  EXPECT_EQ(castNULLABLEINTERVALDAY_int64(-55), -55);
+  EXPECT_EQ(castNULLABLEINTERVALDAY_int64(-1201), -1201);
+
+  // Test castNULLABLEINTERVALYEAR for int and bigint
+  EXPECT_EQ(castNULLABLEINTERVALYEAR_int32(context_ptr, 1), 1);
+  EXPECT_EQ(castNULLABLEINTERVALYEAR_int32(context_ptr, 12), 12);
+  EXPECT_EQ(castNULLABLEINTERVALYEAR_int32(context_ptr, 55), 55);
+  EXPECT_EQ(castNULLABLEINTERVALYEAR_int32(context_ptr, 1201), 1201);
+  EXPECT_EQ(castNULLABLEINTERVALYEAR_int64(context_ptr, 1), 1);
+  EXPECT_EQ(castNULLABLEINTERVALYEAR_int64(context_ptr, 12), 12);
+  EXPECT_EQ(castNULLABLEINTERVALYEAR_int64(context_ptr, 55), 55);
+  EXPECT_EQ(castNULLABLEINTERVALYEAR_int64(context_ptr, 1201), 1201);
+  // validate overflow error when using bigint as input
+  castNULLABLEINTERVALYEAR_int64(context_ptr, INT64_MAX);
+  EXPECT_EQ(context.get_error(), "Integer overflow");
+  context.Reset();
+}
+
 }  // namespace gandiva
