@@ -35,7 +35,7 @@ _default_config_path = _default_arrow_path / "dev" / "tasks" / "tasks.yml"
 
 @click.group()
 @click.option('--github-token', '-t', default=None,
-              envvar="CROSSBOW_GITHUB_TOKEN",
+              envvar=['CROSSBOW_GITHUB_TOKEN', 'GH_TOKEN'],
               help='OAuth token for GitHub authentication')
 @click.option('--arrow-path', '-a',
               type=click.Path(), default=_default_arrow_path,
@@ -150,11 +150,10 @@ def submit(obj, tasks, groups, params, job_prefix, config_path, arrow_version,
     queue.put(job, prefix=job_prefix)
 
     if no_push:
-        click.echo('Branches and commits created but not pushed: `{}`'
-                   .format(job.branch))
+        click.echo(f'Branches and commits created but not pushed: `{job.branch}`')
     else:
         queue.push()
-        click.echo('Pushed job identifier is: `{}`'.format(job.branch))
+        click.echo(f'Pushed job identifier is: `{job.branch}`')
 
 
 @crossbow.command()
@@ -174,7 +173,7 @@ def submit(obj, tasks, groups, params, job_prefix, config_path, arrow_version,
                    'locally. Examples: https://github.com/apache/arrow or '
                    'https://github.com/raulcd/arrow.')
 @click.option('--rc', default=None,
-              help='Relase Candidate number.')
+              help='Release Candidate number.')
 @click.option('--version', default=None,
               help='Release version.')
 @click.option('--verify-binaries', is_flag=True, default=False,
@@ -263,7 +262,7 @@ def render(obj, task, config_path, arrow_version, arrow_remote, arrow_branch,
     for task_name, rendered_files in job.render_tasks().items():
         for path, content in _flatten(rendered_files).items():
             click.echo('#' * 80)
-            click.echo('### {:^72} ###'.format("/".join(path)))
+            click.echo(f"### {'/'.join(path):^72} ###")
             click.echo('#' * 80)
             click.echo(highlight(content))
 
@@ -505,7 +504,17 @@ def download_artifacts(obj, job_name, target_dir, dry_run, fetch,
         if asset is not None:
             path = target_dir / task_name / asset.name
             path.parent.mkdir(exist_ok=True)
-            if not dry_run:
+
+            def need_download():
+                if dry_run:
+                    return False
+                if not path.exists():
+                    return True
+                if path.stat().st_size != asset.size:
+                    return True
+                return False
+
+            if need_download():
                 import github3
                 max_n_retries = 5
                 n_retries = 0
@@ -524,8 +533,8 @@ def download_artifacts(obj, job_name, target_dir, dry_run, fetch,
                     else:
                         break
 
-    click.echo('Downloading {}\'s artifacts.'.format(job_name))
-    click.echo('Destination directory is {}'.format(target_dir))
+    click.echo(f'Downloading {job_name}\'s artifacts.')
+    click.echo(f'Destination directory is {target_dir}')
     click.echo()
 
     report = ConsoleReport(job, task_filters=task_filters)
