@@ -17,10 +17,13 @@
 
 from collections import namedtuple, OrderedDict
 import binascii
+import gzip
 import json
+import math
 import os
 import random
 import tempfile
+import shutil
 
 import numpy as np
 
@@ -175,6 +178,9 @@ class IntegerField(PrimitiveField):
             ('bitWidth', self.bit_width)
         ])
 
+    def _encode_values(self, values):
+        return list(map(int if self.bit_width < 64 else str, values))
+
     def generate_column(self, size, name=None):
         lower_bound, upper_bound = self._get_generated_data_bounds()
         return self.generate_range(size, lower_bound, upper_bound,
@@ -185,7 +191,7 @@ class IntegerField(PrimitiveField):
         values = np.random.randint(lower, upper, size=size, dtype=np.int64)
         if include_extremes and size >= 2:
             values[:2] = [lower, upper]
-        values = list(map(int if self.bit_width < 64 else str, values))
+        values = self._encode_values(values)
 
         is_valid = self._make_is_valid(size)
 
@@ -193,34 +199,21 @@ class IntegerField(PrimitiveField):
             name = self.name
         return PrimitiveColumn(name, size, is_valid, values)
 
+    @property
+    def column_class(self):
+        return PrimitiveColumn
+
 
 # Integer field that fulfils the requirements for the run ends field of REE.
 # The integers are positive and in a strictly increasing sequence
 class RunEndsField(IntegerField):
-    # bit_width should only be one of 16/32/64
     def __init__(self, name, bit_width, *, metadata=None):
+        assert bit_width in (16, 32, 64)
         super().__init__(name, is_signed=True, bit_width=bit_width,
-                         nullable=False, metadata=metadata, min_value=1)
+                         nullable=False, metadata=metadata)
 
-    def generate_range(self, size, lower, upper, name=None,
-                       include_extremes=False):
-        rng = np.random.default_rng()
-        # generate values that are strictly increasing with a min-value of
-        # 1, but don't go higher than the max signed value for the given
-        # bit width. We sort the values to ensure they are strictly increasing
-        # and set replace to False to avoid duplicates, ensuring a valid
-        # run-ends array.
-        values = rng.choice(2 ** (self.bit_width - 1) - 1, size=size, replace=False)
-        values += 1
-        values = sorted(values)
-        values = list(map(int if self.bit_width < 64 else str, values))
-        # RunEnds cannot be null, as such self.nullable == False and this
-        # will generate a validity map of all ones.
-        is_valid = self._make_is_valid(size)
-
-        if name is None:
-            name = self.name
-        return PrimitiveColumn(name, size, is_valid, values)
+    def generate_column(self, size, name=None):
+        raise NotImplementedError("cannot be generated directly")
 
 
 class DateField(IntegerField):
@@ -1157,11 +1150,32 @@ class RunEndEncodedField(Field):
         ]
 
     def generate_column(self, size, name=None):
-        values = self.values_field.generate_column(size)
-        run_ends = self.run_ends_field.generate_column(size)
+        # The `size` of a RunEndEncodedField is the logical length of the
+        # run-end-encoded column, so we choose a number of physical runs
+        # that's smaller.
+        if size > 0:
+            num_runs = np.random.randint(1, math.ceil(size * 0.75))
+            # Generate run ends
+            run_ends = np.random.choice(size - 1, num_runs - 1, replace=False) + 1
+            run_ends.sort()
+            run_ends = np.concat((run_ends, [size]))
+            assert len(run_ends) == num_runs
+            assert len(set(run_ends)) == num_runs
+            assert (run_ends > 0).all()
+            assert (run_ends <= size).all()
+        else:
+            num_runs = 0
+            run_ends = []
+        run_ends_is_valid = self._make_is_valid(num_runs, null_probability=0)
+        run_ends = self.run_ends_field._encode_values(run_ends)
+
+        run_end_column = self.run_ends_field.column_class(
+            self.run_ends_field.name, num_runs, run_ends_is_valid, run_ends)
+        values = self.values_field.generate_column(num_runs)
+
         if name is None:
             name = self.name
-        return RunEndEncodedColumn(name, size, run_ends, values)
+        return RunEndEncodedColumn(name, size, run_end_column, values)
 
 
 class _BaseUnionField(Field):
@@ -1429,7 +1443,7 @@ class File(object):
         self.path = path
 
     def skip_tester(self, tester):
-        """Skip this test for the given tester (such as 'C#').
+        """Skip this test for the given tester (such as '.NET').
         """
         self.skipped_testers.add(tester)
         return self
@@ -1548,8 +1562,7 @@ def generate_duplicate_fieldnames_case():
 def generate_primitive_case(batch_sizes, name='primitive'):
     types = ['bool', 'int8', 'int16', 'int32', 'int64',
              'uint8', 'uint16', 'uint32', 'uint64',
-             'float32', 'float64', 'binary', 'utf8',
-             'fixedsizebinary_19', 'fixedsizebinary_120']
+             'float32', 'float64']
 
     fields = []
 
@@ -1560,7 +1573,19 @@ def generate_primitive_case(batch_sizes, name='primitive'):
     return _generate_file(name, fields, batch_sizes)
 
 
-def generate_primitive_large_offsets_case(batch_sizes):
+def generate_binary_case(batch_sizes, name='binary'):
+    types = ['binary', 'utf8', 'fixedsizebinary_19', 'fixedsizebinary_120']
+
+    fields = []
+
+    for type_ in types:
+        fields.append(get_field(type_ + "_nullable", type_, nullable=True))
+        fields.append(get_field(type_ + "_nonnullable", type_, nullable=False))
+
+    return _generate_file(name, fields, batch_sizes)
+
+
+def generate_large_binary_case(batch_sizes):
     types = ['largebinary', 'largeutf8']
 
     fields = []
@@ -1569,7 +1594,7 @@ def generate_primitive_large_offsets_case(batch_sizes):
         fields.append(get_field(type_ + "_nullable", type_, nullable=True))
         fields.append(get_field(type_ + "_nonnullable", type_, nullable=False))
 
-    return _generate_file('primitive_large_offsets', fields, batch_sizes)
+    return _generate_file('large_binary', fields, batch_sizes)
 
 
 def generate_null_case(batch_sizes):
@@ -1744,11 +1769,14 @@ def generate_recursive_nested_case():
 
 def generate_run_end_encoded_case():
     fields = [
-        RunEndEncodedField('ree16', 16, get_field('values', 'int32')),
-        RunEndEncodedField('ree32', 32, get_field('values', 'utf8')),
-        RunEndEncodedField('ree64', 64, get_field('values', 'float32')),
+        RunEndEncodedField('ree16_int32', 16, get_field('values', 'int32')),
+        RunEndEncodedField('ree32_utf8', 32, get_field('values', 'utf8')),
+        RunEndEncodedField('ree64_float32', 64, get_field('values', 'float32')),
+        RunEndEncodedField('ree16_bool', 64, get_field('values', 'bool')),
+        # Add a non-REE-encoded field to check column size correctness
+        BooleanField('bool'),
     ]
-    batch_sizes = [0, 7, 10]
+    batch_sizes = [0, 7, 20]
     return _generate_file("run_end_encoded", fields, batch_sizes)
 
 
@@ -1885,15 +1913,16 @@ def generate_extension_case():
 def get_generated_json_files(tempdir=None):
     tempdir = tempdir or tempfile.mkdtemp(prefix='arrow-integration-')
 
-    def _temp_path():
-        return
-
     file_objs = [
         generate_primitive_case([], name='primitive_no_batches'),
         generate_primitive_case([17, 20], name='primitive'),
         generate_primitive_case([0, 0, 0], name='primitive_zerolength'),
 
-        generate_primitive_large_offsets_case([17, 20]),
+        generate_binary_case([], name='binary_no_batches'),
+        generate_binary_case([17, 20], name='binary'),
+        generate_binary_case([0, 0, 0], name='binary_zerolength'),
+
+        generate_large_binary_case([17, 20]),
 
         generate_null_case([10, 0]),
 
@@ -1951,23 +1980,23 @@ def get_generated_json_files(tempdir=None):
         # TODO(https://github.com/apache/arrow-nanoarrow/issues/622)
         .skip_tester('nanoarrow')
         # TODO(https://github.com/apache/arrow/issues/38045)
-        .skip_format(SKIP_FLIGHT, 'C#'),
+        .skip_format(SKIP_FLIGHT, '.NET'),
 
         generate_dictionary_unsigned_case()
         .skip_tester('nanoarrow')
         .skip_tester('Java')  # TODO(ARROW-9377)
         # TODO(https://github.com/apache/arrow/issues/38045)
-        .skip_format(SKIP_FLIGHT, 'C#'),
+        .skip_format(SKIP_FLIGHT, '.NET'),
 
         generate_nested_dictionary_case()
         # TODO(https://github.com/apache/arrow-nanoarrow/issues/622)
         .skip_tester('nanoarrow')
         .skip_tester('Java')  # TODO(ARROW-7779)
         # TODO(https://github.com/apache/arrow/issues/38045)
-        .skip_format(SKIP_FLIGHT, 'C#'),
+        .skip_format(SKIP_FLIGHT, '.NET'),
 
         generate_run_end_encoded_case()
-        .skip_tester('C#')
+        .skip_tester('.NET')
         .skip_tester('JS')
         # TODO(https://github.com/apache/arrow-nanoarrow/issues/618)
         .skip_tester('nanoarrow')
@@ -1980,7 +2009,7 @@ def get_generated_json_files(tempdir=None):
         .skip_tester('Rust'),
 
         generate_list_view_case()
-        .skip_tester('C#')     # Doesn't support large list views
+        .skip_tester('.NET')     # Doesn't support large list views
         .skip_tester('JS')
         # TODO(https://github.com/apache/arrow-nanoarrow/issues/618)
         .skip_tester('nanoarrow')
@@ -1992,7 +2021,7 @@ def get_generated_json_files(tempdir=None):
         .skip_format(SKIP_C_SCHEMA, 'C++')
         .skip_format(SKIP_C_ARRAY, 'C++')
         # TODO(https://github.com/apache/arrow/issues/38045)
-        .skip_format(SKIP_FLIGHT, 'C#'),
+        .skip_format(SKIP_FLIGHT, '.NET'),
     ]
 
     generated_paths = []
@@ -2003,3 +2032,25 @@ def get_generated_json_files(tempdir=None):
         generated_paths.append(file_obj)
 
     return generated_paths
+
+
+def generate_gold_files(tester, gold_dir):
+    os.makedirs(gold_dir, exist_ok=True)
+
+    # Generate JSON files
+    files = get_generated_json_files(gold_dir)
+    for f in files:
+        # For each JSON file, convert it to Arrow IPC file and stream
+        json_path = os.path.join(gold_dir, 'generated_' +
+                                 f.name + '.json')
+        arrow_file_path = os.path.join(gold_dir, 'generated_' +
+                                       f.name + '.arrow_file')
+        stream_path = os.path.join(gold_dir, 'generated_' +
+                                   f.name + '.stream')
+        tester.json_to_file(json_path, arrow_file_path)
+        tester.file_to_stream(arrow_file_path, stream_path)
+        # And GZip-compress the JSON file
+        with open(json_path, 'rb') as f_in:
+            with gzip.open(json_path + '.gz', 'wb') as f_out:
+                shutil.copyfileobj(f_in, f_out)
+        os.unlink(json_path)
