@@ -36,6 +36,9 @@ rpmbuild_options=
 if grep -q amazon /etc/system-release-cpe; then
   distribution=$(cut -d ":" -f 5 /etc/system-release-cpe | tr '_' '-')
   distribution_version=$(cut -d ":" -f 6 /etc/system-release-cpe)
+elif grep -q oracle /etc/system-release-cpe; then
+  distribution=oracle-linux
+  distribution_version=$(cut -d ":" -f 5 /etc/system-release-cpe)
 else
   distribution=$(cut -d ":" -f 4 /etc/system-release-cpe)
   distribution_version=$(cut -d ":" -f 5 /etc/system-release-cpe)
@@ -90,7 +93,7 @@ fi
 repositories="/host/repositories"
 repository="${repositories}/${distribution}/${distribution_version}"
 rpm_dir="${repository}/${architecture}/Packages"
-srpm_dir="${repository}/source/SRPMS"
+srpm_dir="${repository}/Source/Packages"
 run mkdir -p "${rpm_dir}" "${srpm_dir}"
 
 # for debug
@@ -126,6 +129,8 @@ run cp \
     /host/tmp/${PACKAGE}.spec \
     rpmbuild/SPECS/
 
+df -h
+
 run cat <<BUILD > build.sh
 #!/usr/bin/env bash
 
@@ -155,6 +160,8 @@ else
   fi
 fi
 
+df -h
+
 if which ccache > /dev/null 2>&1; then
   ccache --show-stats --verbose || :
 fi
